function TabTeX(file,title,label,chead,hrow,fcol,hline,csin,dec,data,fnote,opt,small,ihead,itext)
% TabTex v1.42
% Creates LaTEX code of a table according to user supplied specifications.
% For available layouts and argument specification see below.
% Uses LaTEX package booktabs.sty. 
%
% *Compile tables created by TabTeX using the PrevTeX.m function, see 
% demo below. 
%
%==========================================================================
%
% 1. TABLE TEMPLATE LAYOUT:
%
% TITLE
% ___________________________________________________________________
%     ROW 1       TABLE HEADING (NUMBER OF ROWS AND MULTICOLUMN
%                                               STRUCTURE OPTIONAL)*
%               ---------------------------  ------------------------
%     ROW 2   TABLE HEADING                         TABLE HEADING  
% -------------------------------------------------------------------
%                0.93   0.70   0.73   0.44    90   113   43
%                0.78   0.58   0.91   0.57    75    80   51
%                0.63   0.56   0.86   0.77    30    66   53
%                0.90                                    47
%    FIRST               TABLE BODY (NUMERIC DATA ONLY) 
%    COLUMN                  
%    LABELS      0.58   0.                          64   55 
%  (OPTIONAL)    0.87   0.88   0.87   0.72    35    95   60
% --------------- ADDITIONAL HORIZONTAL LINES IF SPECIFIED (hline)
% ---------BODY SECTION LABELS IF SPECIFIED (ihead,itext)
%                0.88   0.92   0.87   0.72    35    78   50
%                0.94   0.77   0.80   0.14   102   104   37
%                0.74   0.41   0.72   0.68    46    72   49
% -----------------------LINE AUTOMATICALLY ADDED--------------------
%    MEDIAN      0.87   0.63   0.80   0.60    45    80   50  
%   (OPTIONAL)
% ___________________________________________________________________
% NOTES: (script size table endnotes)
% (*) - Horizontal lines are generated automatically
%       Lines in heading can be suppressed 
%           - use 0 instead of 2 whenever in chead.
%==========================================================================
%
% 2. FUNCTION ARGUMENT SPECIFICATION:
%
%--------------------------------------------------------------------------
% Entry | Description                | Type                   | Example   
%--------------------------------------------------------------------------
%       | Name of the output file    | cell array of strings  | {'Tab.tex'}
%       | with resulting TeX code.   | 1 row                  | 
% file  | The subfolder \Tex must    |                        |  or
%       | exist in the current       | output file writen to  | 
%       | MATLAB workspace.          | \Tex subfolder         | {'Tab.txt'}
%--------------------------------------------------------------------------
%       | Title of the table         | cell array of strings  |
%       | environment - apears in    | 1 row                  |
% title | caption field of the       |                        | {'CA deficit'}
%       | 'TeX table environment'    |                        |
%--------------------------------------------------------------------------
%       | Table label, apears        | cell array of strings  |
% label | in the label field         | 1 row                  | {'Table3'}
%       | of 'TeX table environment' |                        |
%--------------------------------------------------------------------------
%       | Table heading specification| matrix with integers   | [0 1 2 2 2;
%       | - matrix specifying heading| number of rows=        |  1 1 1 1 1]
% chead*| layout. Can be suppressed. | number of heading rows |      
%       | 0 - no label in this cell  | number of columns =    |     or
%       | 1 - add label from hrow    | number of columns in   | [] -BLANK 
%       | 2 - merge with previous    | the entire table       |  
%       | See (*) below for details  |                        | 
%--------------------------------------------------------------------------
%       | Table heading labels (TEX) | cell array of strings: | {'$x^{T}$';'$y$'}
%       | - each subsequent entry    | number of fields must  | 
% hrow  | corresponds to subsequent  | agree with the number  |   or
%       | 1s in the chead matrix     | of 1s entered in chead |   
%       | Heading suppressed if BLANK| Read sequentially.     | {} -BLANK
%--------------------------------------------------------------------------
%       | First column non-          |                        | {'BEL';'CAN'}
%       | numeric data/labels.       | cell array of strings  | 
% fcol  | Suppressed if BLANK.       |                        |     or
%       |                            |                        | {} -BLANK
%--------------------------------------------------------------------------
%       | Specifies additional       | vector row numbers     | [2;3]
% hline | horizontal lines           | in ascending order     | adds horiz.
%       | to apear after the         | can't exceed number    | line after
%       | indicated rows             | of rows in the table   | rows 2,3 
%--------------------------------------------------------------------------
%       | Size of each column in     | numeric                | 
% csin  | inches                     | vector                 | [2;1.1;0.5]
%       |                            |                        |
%--------------------------------------------------------------------------
%       | Number of deciman places   | numeric integers       | 
% dec   | in each column of data     | vector                 | [2;1]
%       |                            |                        |
%--------------------------------------------------------------------------
%       | Matrix with numeric data   | data(table rows,       |
% data  | needs to be formatted (num.|         table columns) | [0.8 0.2;
%       | of decimal digits adjusted)|                        |  1.1 2.0]
%       | See example call below.    |                        |  
%--------------------------------------------------------------------------
%       | Text of the footnote under | cell array of strings  | {'HP filtered'}
% fnote | in script size the table   |                        |     or
%       | Suppressed if blank {}     |                        | {} -BLANK
%--------------------------------------------------------------------------
% Formating option specification : opt numeric vector
%--------------------------------------------------------------------------
%       | =2 adds the last row with  |                        |
%       |  column averages generated |                        |
%       |  automatically             | opt is a numeric vector| [0 0 0]
% opt(1)| =1 adds the last row with  | of size 3              | 
%       |  column medians generated  |                        |  
%       |  automatically,            |                        |
%       | =0 no median or average    |                        |
%--------------------------------------------------------------------------
%       | Sets up minipage           | opt is a numeric vector| [0 0 0]
%       | format environment for the | of size 3              | 
% opt(2)| table. Suppressed if BLANK.| manual setting expressed 
%       | 0  - autosize minipage     | as fraction of         |
%       | >0 - manual size           | total linewidth        | 
%--------------------------------------------------------------------------
%       | If 0 removes separating    | opt is a numeric vector| [0 0 0]
%       | line between table and     | of size at least 3     | 
% opt(3)|                            |                        | 
%       | 1 removes the line         |                        |
%--------------------------------------------------------------------------
%       | If 0 default setting       | opt is a numeric vector| [0 0 0]
% opt(4)| 1 removes lines from       | of size at least 3     | 
%       | table heading specification|                        | 
%--------------------------------------------------------------------------
%
% FOR EXTENDED TABLE MODE** SUPPLY ALSO THE FOLLOWING ARGUMENTS
%
% **EXTENDED TABLE MODE : tables with additional body section headings
%
%--------------------------------------------------------------------------
%       | resize table body to       | numeric vector         | [1 2]; 
% small | small font                 | specifying column      | if supplied
%       | [] - makes all             | numbers to be resized  | but empty [];
%       | data entry be in small font| to small font          | all entries 
%       |                            |                        | small
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%       | Defines position of body   | vector,                |
%       | section headings.          | of integers            |
%       | Appears above listed       | specifies              |
% ihead | rows.                      | row position of the    |  [2;3];
%       |                            | label - to appear      |   
%       |                            | above the data row    |
%       |                            | of the specified number|
%--------------------------------------------------------------------------
%       | Body section headings      | cell array of strings  | {'label1';
% itext | - positioned by ihead      | must be specified      | 'label2'}
%       |                            | if ihead specified     |
%--------------------------------------------------------------------------

%(*) chead - Table heading layout specification:
%            Number of rows  determines the number of rows to be included in 
%            the heading of the table.
%            Number of columns must agree with the number of
%            columns in the entire table. 
%            Each entry of 'chead' specifies labeling options of the 
%            heading cells. Heading can be suppressed if left BLANK: chead=[];.
%
%            Entries in chead can take integer values from 0 to 2 :
%             - 1 inputs subsequent label from hrow, 0 leaves cell blank, 
%             - 2 builds a multicolumn object that extends previous labels
%               to all subsequent cells with this entry (merge current cell
%               with previous and extend the label throughout). 
%
%            Supporting horizontal lines are created automatically - see 
%            table template description how to suppress this option. 
%==========================================================================
% 3. DEMO
%
% (copy-paste to .m demo code and run, 
%  functions TabTeX, PrevTeX must be present in the current workspace,
%  Tex subfolder exist the current Matlab workspace)
%
% % Create sample table 1
% clc; clear
% file = {'Sample1.txt'};
% title= {'Here goes the title'};
% label= {'Sample'};
% chead= [0 1 2 2 2 1 2 2;
%         1 1 1 1 1 1 1 1];
% hrow = {'Correlation','Relative volatility (\%)',...
%         'Area','$p_{x},p_{m}$','$p_{x},x$','$p_{m},x$','$p,x$',...
%         '$p_{x}$','$p_{m}$','$p$'};
% data = ones(9,7); 
% data(3:5,4:6)=-1.008; %rounding and number alignment demo
% ihead=[1,4];
% itext={'\textit{Green countries}','\textit{Blue countries}'};
% fcol = {'\quad Belgium \footnote{Footnotes can be added too.}';'\quad Canada';'\quad Switzerland';'\quad France';...
%           '\quad Germany';'\quad Italy';'\quad Japan';'\quad Netherlands';'\quad US'};   
% hline= [3];
% csin = [1.0;.5;.5;.5;.5;.5;.5;.5]; 
% dec  = [     2; 2; 2; 2; 1; 1; 1]; 
% opt(1)=1; opt(2)=0;
% fnote= {'Notes: Based on logged \& HP filtered quarterly data for the period 1980:1-2000:2. Sources listed at the end of the tables.'};
% TabTeX(file,title,label,chead,hrow,fcol,hline,csin,dec,data,fnote,opt,ihead,itext);
% 
% % Create sample table 2
% 
% file = {'Sample2.txt'};
% title= {'The same but with suppressed fcol*'};
% label= {'Sample2'};
% chead= [1 2 2 2 1 2 2;
%         1 1 1 1 1 1 1];
% hrow = {'Correlation','Relative volatility (\%)',...
%         '$p_{x},p_{m}$','$p_{x},x$','$p_{m},x$','$p,x$',...
%         '$p_{x}$','$p_{m}$','$p$'};
% data = ones(9,7); 
% data(3:5,4:6)=-1.008; %rounding and number alignment demo
% ihead=[1,4];
% itext={'\textit{Green countries}','\textit{Blue countries}'};
% fcol = {};   
% hline= [3];
% csin = [.5;.5;.5;.5;.5;.5;.5]; 
% dec  = [ 2; 2; 2; 2; 1; 1; 1]; 
% opt(1)=1; opt(2)=0;
% fnote= {'*TabTeX function argument fcol has been left blank in this case.'};
% TabTeX(file,title,label,chead,hrow,fcol,hline,csin,dec,data,fnote,opt,ihead,itext);
% PrevTeX({'Sample1.txt','Sample2.txt'}) %compile and preview both tables
%% PrevTeX creates AllTabs.tex, which is the final 
%%          output file with combined codes of Sample 1 and 2.
%==========================================================================
% 4. COMMENTS: 
%
% a. Must NOT use TeX restricted symbols '%,$,&...' in user supplied strings,
%    unless proceeded with '\'. Causes LaTeX compilation Error.  
% b. PrevTeX function: pdflatex and Adobe Acrobat Reader must be 
%    installed and added to system path, Acrobat Reader must be closed.
% c. Tex subfolder in the current Matlab workspace must exist.
% e. If footnote under the table is too short with respect 
%    to the width of the table use manual mode of opt(2)
%    or adjust the csin specification - csin is too small for
%    at least one column (csin sets lower bound on size, upper bound 
%    depends on the size of cell content). Can also change the formula in 
%    line 283 of TabTeX code. 
%
% (c) by Lukasz Drozd, Jaromir Nosal
%     February 17, 2006

cd Tex %enter into Tex subfolder (must exist!!!)
cc  = size(data); %determine size specifications, check consistency
mm  = max(size(fcol));
hh  = size(chead);
dd  = max(size(dec));
ss  = max(size(csin));

%Check for consistency in user supplied arguments (size, type)
if isempty(opt) || max(size(opt))<2 
    error('TabTeX.m ERROR message: Illegal specification found in ''opt'' ')
end
if ~iscell(fcol)
    cd ..
    error('TabTeX.m ERROR message: Illegal specification found in ''fcol'' ')
end

if isempty(fcol) %first column specification
    fsup=0;cc(1,2)=cc(1,2);cc(1,1)=cc(1,1)+min(opt(1),1)*1; %reset size specification if fcol empty
    %fsup =0 the first non-numeric data column is suppressed
else  
    if mm~=cc(1,1)
        cd ..
        error('TabTeX.m ERROR message: Size of fcols and data must agree')
    end
    fsup=1;cc(1,2)=cc(1,2)+1;cc(1,1)=cc(1,1)+min(opt(1),1)*1;
end 

if cc(1,2)~=dd+1*fsup || cc(1,2)~=ss || hh(1,2)~=dd+1*fsup %column specification consistency check
   cd ..
   error('TabTeX.m ERROR message: Inconsistent column specification found in ''csin'', ''dec'' or ''chead'' ')
end

if isnumeric(dec) && isnumeric(data)
else
    cd ..
    error('TabTeX.m ERROR message: Illegal function argument specification found in ''data/dec'' ')
end

linesup=0;
if max(size(opt))>=4 %ARE lines in table heading requested to be suppressed?
    if opt(4)==1; linesup=1; else linesup=0; end
end

%_________________________________________________________________
%SET TABLE MODE depending on number of arguments supplied by user

txt=zeros(cc(1,1),1);
sll(1:cc(1,2))=  cellstr('');

if  nargin    == 12 %REGULAR TABLE MODE 1 (normal font)

elseif nargin == 13 %REGULAR TABLE MODE 2 (small font)
    
    if isempty(small); sll(1:cc(1,2))=cellstr('\small ');
    else counter=1; 
            tx=max(size(small));
            while( counter<=tx ) %position body section headings (constructs variable txt)
                for i=1:cc(1,2)
                    if small(counter)==i; sll(i)=cellstr('\small '); end 
                end
                counter=counter+1;
            end  
    end    
    
%Process user requested options in the extended table mode

elseif nargin == 14 %EXTENDED TABLE MODE 1 (normal font)
    
       itext=ihead; ihead=small;
       if ~isempty(ihead); tx=max(size(ihead)); counter=1;
           if max(size(ihead))~=max(size(itext))
              error('TabTeX.m ERROR message: Size of ihead and itext must agree')
           end
           while( counter<=tx ) %position body section headings (constructs variable txt)
               for i=1:cc(1,1)
                    if ihead(counter)==i; txt(i)=txt(i)+1; end %allow for multiple entries in ihead by cumulating
               end
               counter=counter+1;
           end
        end
        
elseif nargin == 15 %EXTENDED TABLE MODE 2 (small font)
    
       if ~isempty(ihead)
           if max(size(ihead))~=max(size(itext))
              error('TabTeX.m ERROR message: Size of ihead and itext must agree')
           end
       if isempty(small); sll(1:cc(1,2))=cellstr('\small ');
       else counter=1; 
             tx=max(size(small));
           while( counter<=tx ) %position body section headings (constructs variable txt)
               for i=1:cc(1,2)
                  if small(counter)==i; sll(i)=cellstr('\small '); end 
               end
               counter=counter+1;
           end  
       end
           counter=1; tx=max(size(ihead));
           while( counter<=tx )
               for i=1:cc(1,1)
                    if ihead(counter)==i; txt(i)=txt(i)+1; end  
               end
               counter=counter+1;
           end
       end
end

%Position user requested horizontal lines (define index variable hl)
hl=zeros(cc(1,1),1);
if ~isempty(hline)
   tx=max(size(hline)); counter=1;
   while( counter<=tx )
    for i=1:cc(1,1)-1*(1-min(opt(1),1))
         if hline(counter)==i 
             hl(i)=1; 
         end  
    end
   counter=counter+1;
   end
end

%Size variables
% 
%cc(1,1) num of rows excluding table heading, 
%cc(1,2) num of columns (including fcol unless suppressed by user)
%

%check if neccessary arguments supplied/are numeric
if iscell(file) && iscell(title) && iscell(label) && isnumeric(csin) %check errors
else
    cd ..
     error('TabTeX.m ERROR message: Illegal argument specification found in ''file/title/label'' ')
end

%Generate TeX code________________________________________________

% (A) table environment

fid = fopen( char(file(1)) ,'w');

fprintf(fid,'\\begin{table}\n');

% (B) minipage environment

if ~isempty(opt(2))
    switch opt(2)
        case 0
            fprintf(fid,'\\centering\n');
            fprintf(fid,'\\begin{minipage}[b]{%.2f\\linewidth}\n',(sum(csin)+.162*cc(1,2))/6.475);
        otherwise
            fprintf(fid,'\\centering\n');
            fprintf(fid,'\\begin{minipage}[b]{%.2f\\linewidth}\n',opt(2));
    end
end

% (C) Table label and caption

fprintf(fid,'\\label{%s}\n'  , char(label(1)));
fprintf(fid,'\\caption{%s}\n', char(title(1)));
fprintf(fid,'\\smallskip\n');
fprintf(fid,'\\begin{tabular}{p{%gin}',csin(1));

% (D) Table heading (top)

for i=2:cc(1,2)
    fprintf(fid,'p{%gin}',csin(i));
end
fprintf(fid,'} \n \\toprule \n');
cursor=1;

if ~isempty(chead) && ~isempty(hrow)
stop   =zeros(cc(1,2),1);
start  =zeros(cc(1,2),1);
mc=1;
for i=1:hh(1,1)
    for j=1:cc(1,2)-1
        switch chead(i,j)
        case 0
            fprintf(fid,' & ');
        case 1
            if chead(i,j+1)==0 || chead(i,j+1)==1 
                        %enter if and only if subsequent entry is 
                        %0 (no label) or 1 (put label)
               fprintf(fid,'%s & ',char(hrow(cursor)));
               cursor=cursor+1; %cursor in hrow
             end 
        case 2
            if chead(i,j-1)==1
                mc=1;
                for z=j:cc(1,2)
                    if chead(i,z)==2
                       mc=mc+1;
                    else 
                       break
                    end
                end
                if j-1+mc-1==cc(1,2)
                    fprintf(fid,'\\multicolumn{%g}{l}{%s}',mc,char(hrow(cursor)));
                else
                    fprintf(fid,'\\multicolumn{%g}{l}{%s} & ',mc,char(hrow(cursor)));
                end
                start(j)=j-1; stop(j)=j-1+mc-1;
                %keep track of the multicolumn structure to generate horizontal
                %lines later
                cursor=cursor+1;
            end
        otherwise
            cd ..
              error('TabTeX.m ERROR message: Illegal specification found in ''chead'' ')
              
        end %end of switch
    end %end of inner for
    
    %last two cells merged (a special case)
    switch chead(i,cc(1,2))
        case 0
        case 1
            fprintf(fid,'%s',char(hrow(cursor)));
            cursor=cursor+1;
        case 2
            if chead(i,cc(1,2)-1)==1
                mc=2;
                fprintf(fid,'\\multicolumn{%g}{l}{%s}',mc,char(hrow(cursor)));
                start(cc(1,2))=cc(1,2)-1; stop(cc(1,2))=cc(1,2);
                %keep track of multicolumn to generate horizontal
                %lines later
                cursor=cursor+1;
            end
        otherwise
              cd ..
              error('TabTeX.m ERROR message: Illegal specification found in ''chead'' ')
    end 
    fprintf(fid,'\\\\ \n ');
    
    %add horizontal lines to the heading (suppress if user requested so...)
    if mc>1 && linesup==0
        for j=1:cc(1,2)
            if start(j)~=stop(j)
                fprintf(fid,'\\cmidrule(r){%g-%g} \n',start(j),stop(j));
            end
        end
    end
    stop   =zeros(cc(1,2),1);
    start  =zeros(cc(1,2),1);
    mc     =1;
end %outer loop ends here
fprintf(fid,'\\midrule \n');
end

% (E) Table body with data

cursor=1;    
    for i=1:cc(1,1)-min(opt(1),1)*1
                fs='';
            if txt(i)>=1      %enter body section labels if provided
               for k=1:txt(i) %deal with multiple entries in ihead, e.g. [.. 5 5 5 ..]
                       fprintf(fid,'\n \\multicolumn{%g}{l}{%s} \\\\ \n',cc(1,2)-1*fsup,char(itext(cursor)));
                       cursor=cursor+1;
                end
            end
            if fsup==1; fprintf(fid,'%s %s ',char(sll(1)),char(fcol(i))); fs='&' ;end %add first column label unless suppressed
            for j=1:cc(1,2)-1*fsup
                if data(i,j)>=0
                    FMT=char(['%s \\ %s %.',num2str(round(dec(j))),'f']);
                    fprintf(fid,FMT,fs,char(sll(j+1*fsup)),data(i,j));
                    fs='&';
                else
                    FMT=char(['%s %s %.',num2str(round(dec(j))),'f']);
                    fprintf(fid,FMT,fs,char(sll(j+1*fsup)),data(i,j));
                    fs='&';
                end
            end
            fprintf(fid,'\\\\ \n ');
            if hl(i)==1 %add horizontal line if requested by user
               fprintf(fid,'\\midrule \n');
            end    
    end         

% (F) Last row with automatically computed statistics

if fsup==1 %execute last row statistics only first column labels not suppressed
    
switch opt(1)
    case 0
            %skip
    case 1
            %compute median (last row)
            fprintf(fid,'\\midrule\n');
                    fprintf(fid,'\\sc Median');
                    for j=1:cc(1,2)-1*fsup
                    if median(data(:,j))>=0 
                        FMT=char(['&\\ %s %.', num2str(round(dec(j))),'f']);
                        fprintf(fid,FMT,char(sll(j+1*fsup)),median(data(:,j))); 
                    else
                        FMT=char(['& %s %.', num2str(round(dec(j))),'f']);
                        fprintf(fid,FMT,char(sll(j+1*fsup)),median(data(:,j))); 
                    end
                    end
            fprintf(fid,'\\\\ \n ');  
    case 2
            %compute average (last row)
            fprintf(fid,'\\midrule\n');
                    fprintf(fid,'\\sc Average');
                    for j=1:cc(1,2)-1*fsup
                    if mean(data(:,j))>=0
                        FMT=char(['&\\ %s %.', num2str(round(dec(j))),'f']);
                        fprintf(fid,FMT,char(sll(j+1*fsup)),mean(data(:,j))); 
                    else
                        FMT=char(['& %s %.', num2str(round(dec(j))),'f']);
                        fprintf(fid,FMT,char(sll(j+1*fsup)),mean(data(:,j))); 
                    end
                    end
            fprintf(fid,'\\\\ \n ');      
        otherwise
            cd ..
              error('TabTeX.m ERROR message: Illegal argument specification found in ''opt(1)'' ')
end  
end %fsup

% (G) Close table environment, add endnotes under the table

fprintf(fid,'\n');
if max(size(opt))>=3
    if opt(3)==1;fprintf(fid,'\\bottomrule'); else fprintf(fid,'\\bottomrule \\addlinespace'); end
else
    fprintf(fid,'\\bottomrule \\addlinespace'); 
end

fprintf(fid,'\\end{tabular} \n');
if ~isempty(fnote)
    fprintf(fid,'\\newline \n');
            fprintf(fid,'\\scriptsize %s\n',char(fnote(1)));
end
if ~isempty(opt)
    fprintf(fid,'\\end{minipage}\n');
end
fprintf(fid,'\\end{table} \n');
fclose(fid); %close file
cd .. % go back to Matlab workspace path



